function r = binornd2(n,p)
% binornd stripped from error checks and extra parameters
% with quicker procedure for small p-values
% BINORND Random arrays from the binomial distribution.
%   R = BINORND(N,P,MM,NN) returns an array of random numbers chosen from a
%   binomial distribution with parameters N and P.  The size of R is the
%   common size of N and P if both are arrays.  If either parameter is a
%   scalar, the size of R is the size of the other parameter.
%
%   R = BINORND(N,P,MM,NN,...) or R = BINORND(N,P,[MM,NN,...]) returns an
%   MM-by-NN-by-... array.
%
%   See also BINOCDF, BINOINV, BINOPDF, BINOSTAT, RANDOM.

%   BINORND generates values using the definition of the binomial
%   distribution, as a sum of Bernoulli random variables.  See Devroye,
%   Lemma 4.1 on page 428, method on page 524.

%   References:
%      [1]  Devroye, L. (1986) Non-Uniform Random Variate Generation,
%           Springer-Verlag.

%   Copyright 1993-2004 The MathWorks, Inc.
%   $Revision: 2.11.4.7 $  $Date: 2004/12/06 16:37:06 $

smallPlimit = 0.01;
smallPlimitN = 100;
memoryLimit = 5e7;

if numel(n)>1
	sizeOut = size(n);
else
	sizeOut = size(p);
end
ndimsOut = numel(sizeOut);

% Handle the scalar params case efficiently
if prod(sizeOut)==1 % scalar params
	conjP = p>0.5;
	if conjP, p = 1-p; end
	if (p<=smallPlimit && n>smallPlimitN) || n>memoryLimit
		r = binoSmallP(n,p);
	else
		r = sum(rand(n,1) < p);
	end
	if conjP, r = n-r; end

	% Handle the scalar n case efficiently 
elseif isscalar(n)
	p=p(:);
	conjP = p>0.5;
	p(conjP) = 1-p(conjP);
	r = zeros(sizeOut);
	smallP = ((p<=smallPlimit) & (n>smallPlimitN)) | n>memoryLimit;
	smallCount = sum(smallP);
	if smallCount>0
		for i=find(smallP)'
			r(i) = binoSmallP(n,p(i));
		end
	end
	if smallCount<numel(p)
		if n*numel(p)<=memoryLimit % avoid memory depletion
			r(~smallP) = sum( rand(numel(p)-smallCount,n) < repmat(p(~smallP), 1, n), 2);
		else
			for i=find(~smallP)'
				r(i) = sum(rand(1,n)<p(i));
			end
		end
	end
	r(conjP) = n-r(conjP);
% Handle the non-scalar params case
else
	if isscalar(p), p = repmat(p, sizeOut); end
	conjP = p(:)>0.5;
	p(conjP) = 1-p(conjP);
	r = zeros(sizeOut);
	smallP = ((p(:)<=smallPlimit) & (n(:)>smallPlimitN)) | n(:)>memoryLimit;
	smallCount = sum(smallP);
	if smallCount>0
		for i=find(smallP)'
			r(i) = binoSmallP(n(i),p(i));
		end
	end
	if smallCount<numel(p)
		ntemp = n(~smallP);
		ptemp = p(~smallP);
		rtemp = zeros(size(ntemp));
		for i = 1:max(ntemp)
			k = find(ntemp >= i);
			rtemp(k) = rtemp(k) + (rand(size(k)) < ptemp(k));
		end
		r(~smallP) = rtemp;
	end
	r(conjP) = n(conjP)-r(conjP);
end

function r=binoSmallP(n,p)
r=0;
cumP=0;
lnPk = n*log(1-p); % log for precision, and avoiding zero Pk
while log(1-rand) > lnPk - log(1-cumP) % k is larger? "(1-rand)" instead of just "rand" to avoid error in some rand.m implementations
	r=r+1;
	cumP = cumP+exp(lnPk);
	lnPk = lnPk + log((n-r+1)/r*p/(1-p));
end
